@@ -12,7 +12,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # generator.py
-# Copyright (C) 2020-2021 Fracpete (fracpete at gmail dot com)
+# Copyright (C) 2020-2023 Fracpete (fracpete at gmail dot com)
 
 import argparse
 import fnmatch
@@ -28,7 +28,8 @@ logger = logging.getLogger("kodi.generator")
 
 
 def generate(dir, idtype="imdb", recursive=True, pattern="*.imdb", delay=1, dry_run=False, overwrite=False,
-             language="en", fanart="none", fanart_file="folder.jpg", interactive=False, episodes=False):
+             language="en", fanart="none", fanart_file="folder.jpg", interactive=False, episodes=False,
+             ua="Mozilla"):
     """
     Traverses the directory and generates the .nfo files.
 
@@ -56,12 +57,14 @@ def generate(dir, idtype="imdb", recursive=True, pattern="*.imdb", delay=1, dry_
     :type interactive: bool
     :param episodes: whether to generate episode information as well
     :type episodes: bool
+    :param ua: User agent for requests
+    :type ua: str
     """
 
     dirs = []
     determine_dirs(dir, recursive, dirs)
     dirs.sort()
-    logger.info("Dirs: %s" % ",".join(dirs))
+    logger.info("# dirs: %d" % len(dirs))
     if interactive:
         delay = 0
 
@@ -74,11 +77,6 @@ def generate(dir, idtype="imdb", recursive=True, pattern="*.imdb", delay=1, dry_
             xml_path = os.path.join(d, os.path.splitext(id_filename)[0] + ".nfo")
             logger.info("ID file: %s" % id_path)
 
-            write_file = True
-            if not overwrite and os.path.exists(xml_path):
-                write_file = False
-                logger.info(".nfo file already exists, skipping")
-
             id = read_id(id_path)
             logger.info("ID: %s" % id)
 
@@ -88,28 +86,25 @@ def generate(dir, idtype="imdb", recursive=True, pattern="*.imdb", delay=1, dry_
                 else:
                     break
 
-            try:
-                if idtype == "imdb":
-                    doc = generate_imdb(id, language=language, fanart=fanart, fanart_file=fanart_file,
-                                        xml_path=xml_path, episodes=episodes, path=d, overwrite=overwrite,
-                                        dry_run=dry_run)
-                else:
-                    logger.critical("Unhandled ID type: %s" % idtype)
-                    return
-                xml_str = doc.toprettyxml(indent="  ")
-                if dry_run:
-                    print(xml_str)
-                elif write_file:
-                    logger.info("Writing .nfo file: %s" % xml_path)
-                    with open(xml_path, "w") as xml_file:
-                        xml_file.write(xml_str)
-            except Exception:
-                logger.info(traceback.format_exc())
+            if os.path.exists(xml_path) and not overwrite:
+                logger.info(".nfo file already exists, skipping: %s" % xml_path)
 
-            if interactive and not proceed():
-                break
-            if delay > 0:
-                time.sleep(delay)
+            if overwrite or not os.path.exists(xml_path):
+                try:
+                    if idtype == "imdb":
+                        generate_imdb(id, language=language, fanart=fanart, fanart_file=fanart_file,
+                                      xml_path=xml_path, episodes=episodes, path=d, overwrite=overwrite,
+                                      dry_run=dry_run, ua=ua)
+                    else:
+                        logger.critical("Unhandled ID type: %s" % idtype)
+                        return
+                except Exception:
+                    logger.info(traceback.format_exc())
+
+                if interactive and not proceed():
+                    break
+                if delay > 0:
+                    time.sleep(delay)
 
 
 def main(args=None):
@@ -131,14 +126,15 @@ def main(args=None):
     parser.add_argument("--pattern", metavar="GLOB", dest="pattern", required=False, default="*.imdb", help="the pattern for the files that contain the movie IDs")
     parser.add_argument("--delay", metavar="SECONDS", dest="delay", type=int, required=False, default=1, help="the delay in seconds between web queries (to avoid blacklisting)")
     parser.add_argument("--preferred_language", metavar="LANG", dest="language", required=False, default="en", help="the preferred language for the titles (ISO 639-1, see https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes)")
-    parser.add_argument("--fanart", dest="fanart", choices=["none", "download", "use-existing"], default="none", required=False, help="how to deal with fan-art")
+    parser.add_argument("--fanart", dest="fanart", choices=["none", "download", "download-missing", "use-existing"], default="none", required=False, help="how to deal with fan-art")
     parser.add_argument("--fanart_file", metavar="FILE", dest="fanart_file", default="folder.jpg", required=False, help="when downloading or using existing fanart, use this filename")
-    parser.add_argument("--episodes", action="store_true", dest="episodes", required=False, help="whether to generte .nfo files for episdes as well")
+    parser.add_argument("--episodes", action="store_true", dest="episodes", required=False, help="whether to generte .nfo files for episodes as well")
     parser.add_argument("--dry_run", action="store_true", dest="dry_run", required=False, help="whether to perform a 'dry-run', ie only outputting the .nfo content to stdout but not saving it to files")
     parser.add_argument("--overwrite", action="store_true", dest="overwrite", required=False, help="whether to overwrite existing .nfo files, ie recreating them with freshly retrieved data")
     parser.add_argument("--verbose", action="store_true", dest="verbose", required=False, help="whether to output logging information")
     parser.add_argument("--debug", action="store_true", dest="debug", required=False, help="whether to output debugging information")
     parser.add_argument("--interactive", action="store_true", dest="interactive", required=False, help="for enabling interactive mode")
+    parser.add_argument("--user-agent", "--ua", type=str, required=False, default="Mozilla", help="User agent for HTTP requests")
     parsed = parser.parse_args(args=args)
     # interactive mode turns on verbose mode
     if parsed.interactive and not (parsed.verbose or parsed.debug):
@@ -154,7 +150,7 @@ def main(args=None):
     generate(dir=parsed.dir, idtype=parsed.type, recursive=parsed.recursive, pattern=parsed.pattern,
              dry_run=parsed.dry_run, overwrite=parsed.overwrite, language=parsed.language,
              fanart=parsed.fanart, fanart_file=parsed.fanart_file, interactive=parsed.interactive,
-             episodes=parsed.episodes)
+             episodes=parsed.episodes, ua=parsed.user_agent)
 
 
 def sys_main():
